<?php
namespace Gath\Product\Hydrator\Strategy;

use InvalidArgumentException;
use Laminas\Hydrator\Strategy\StrategyInterface;

/**
 * Pretvara Gath DataLinkove i ID-eve i obratno.
 *
 * @package Gath\Product\Hydrator\Strategy
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/gath-product
 */
final class ProductIdStrategy implements StrategyInterface
{
    const ID_OFFSET = 1e7;
    const ID_OFFSET_SIMPLE = self::ID_OFFSET;
    const ID_OFFSET_GROUPED = self::ID_OFFSET_SIMPLE + self::ID_OFFSET;
    const ID_OFFSET_VARIABLE = self::ID_OFFSET_GROUPED + self::ID_OFFSET;
    const ID_OFFSET_VARIATION = self::ID_OFFSET_VARIABLE + self::ID_OFFSET;
    const ID_OFFSET_TIRES = self::ID_OFFSET_VARIATION + self::ID_OFFSET;

    private static array $typeMap = [
        'tire'      => self::ID_OFFSET_TIRES,
        'variation' => self::ID_OFFSET_VARIATION,
        'variable'  => self::ID_OFFSET_VARIABLE,
        'grouped'   => self::ID_OFFSET_GROUPED,
        'simple'    => self::ID_OFFSET_SIMPLE,
    ];

    /**
     * Converts the given value so that it can be extracted by the hydrator.
     *
     * @param mixed $value The original value.
     * @param object|null $object
     * @return array
     */
    public function extract($value, ?object $object = null): array
    {
        foreach (array_keys(self::$typeMap) as $type) {
            if ($value > self::$typeMap[$type]) {
                return [
                    'id' => $value,
                    'type' => $type,
                    'data_link' => $value - self::$typeMap[$type],
                ];
            }
        }

        return [];
    }

    /**
     * Converts the given value so that it can be hydrated by the hydrator.
     *
     * @param mixed $value
     * @param array|null $data
     *
     * @return array
     */
    public function hydrate($value, ?array $data): array
    {
        $type = $data['type'] ?? 'simple';

        if (!isset(self::$typeMap[$type])) {
            throw new InvalidArgumentException(sprintf(
                "Ne znam pretvoriti id za proizvod tipa '%s'.", $type
            ));
        }

        $offset = self::$typeMap[$type];

        return [
            'id' => intval($value) + $offset,
            'type' => $type,
            'data_link' => $value,
        ];
    }
}
