<?php
namespace GathTest\Product;

use Ak\Test\AbstractTestCase;
use Gath\Product\ConfigProvider;
use Gath\Product\ProductInterface;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;
use Webshop\Product\ProductGatewayInterface;

final class ProductGatwayTest extends AbstractTestCase
{
    protected array $providers = [
        \Laminas\Hydrator\ConfigProvider::class,
        \Gath\ConfigProvider::class,
        ConfigProvider::class,
    ];

    /**
     * @return ProductGatewayInterface
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    protected function getGateway(): ProductGatewayInterface
    {
        $container = $this->getContainer([
            'gath' => [
                'base_uri' => 'http://servisi:58925',
            ],
        ]);

        return $container->get(ProductGatewayInterface::class);
    }

    /**
     * @return void
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function testCanGetProductsById()
    {
        $gateway = $this->getGateway();
        /** @var ProductInterface $product */
        $product = $gateway->find(10369072);

        $this->assertInstanceOf(ProductInterface::class, $product);
        $this->assertEquals('3592_K', $product->getSku());
        $this->assertEquals('simple', $product->getType());
        $this->assertEquals(10369072, $product->getId());
        $this->assertEquals(369072, $product->getDataLink());
        $this->assertEquals('3592', $product->getCatalogNumber());
    }
}
